{% extends "tournamentcontrol/competition/runsheet.html" %}

{% block detail_link %}
	<a href="{% url 'admin:fixja:competition:season:division:stage:match:detail' match.stage.division.season.competition.pk match.stage.division.season.pk match.stage.division.pk match.stage.pk match.pk %}">
		<span class="glyphicon glyphicon-stats" aria-hidden="true"></span>
	</a>
{% endblock %}
