# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-07-06 12:00
from __future__ import unicode_literals

import touchtechnology.common.db.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('content', '0004_remove_placeholderkeywordargument'),
    ]

    operations = [
        migrations.AlterField(
            model_name='page',
            name='last_modified',
            field=models.DateTimeField(auto_now=True),
        ),
        migrations.AlterField(
            model_name='pagecontent',
            name='last_modified',
            field=models.DateTimeField(auto_now=True),
        ),
        migrations.AlterField(
            model_name='redirect',
            name='active',
            field=touchtechnology.common.db.models.BooleanField(default=True, verbose_name='Active'),
        ),
        migrations.AlterField(
            model_name='redirect',
            name='permanent',
            field=touchtechnology.common.db.models.BooleanField(default=False, verbose_name='Permanent'),
        ),
    ]
